# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import os
import tempfile
import unittest2

from oslo import messaging
from oslo.config import cfg
from oslo.messaging import transport
import pkg_resources as pkg
from stevedore import driver

from mistral.openstack.common import importutils


# We need to make sure that all configuration properties are registered.
importutils.import_module("mistral.config")

from mistral.db.sqlalchemy import api as db_api
from mistral.openstack.common import log as logging
from mistral.openstack.common.db.sqlalchemy import session
from mistral import version
from mistral.engine import client
from mistral.engine.scalable import engine
from mistral.engine.scalable.executor import server


RESOURCES_PATH = 'tests/resources/'
LOG = logging.getLogger(__name__)


def get_resource(resource_name):
    return open(pkg.resource_filename(
        version.version_info.package,
        RESOURCES_PATH + resource_name)).read()


def get_fake_transport():
    # Get transport here to let oslo.messaging setup default config
    # before changing the rpc_backend to the fake driver; otherwise,
    # oslo.messaging will throw exception.
    messaging.get_transport(cfg.CONF)
    cfg.CONF.set_default('rpc_backend', 'fake')
    url = transport.TransportURL.parse(cfg.CONF, None, None)
    kwargs = dict(default_exchange=cfg.CONF.control_exchange,
                  allowed_remote_exmods=[])
    mgr = driver.DriverManager('oslo.messaging.drivers',
                               url.transport,
                               invoke_on_load=True,
                               invoke_args=[cfg.CONF, url],
                               invoke_kwds=kwargs)
    return transport.Transport(mgr.driver)


class BaseTest(unittest2.TestCase):
    def setUp(self):
        super(BaseTest, self).setUp()

        # TODO: add whatever is needed for all Mistral tests in here

    def tearDown(self):
        super(BaseTest, self).tearDown()

        # TODO: add whatever is needed for all Mistral tests in here

    def _assert_single_item(self, items, **props):
        return self._assert_multiple_items(items, 1, **props)[0]

    def _assert_multiple_items(self, items, count, **props):
        def _matches(item, **props):
            for prop_name, prop_val in props.iteritems():
                v = item[prop_name] if isinstance(item, dict) \
                    else getattr(item, prop_name)

                if v != prop_val:
                    return False

            return True

        filtered_items = filter(lambda item: _matches(item, **props), items)

        found = len(filtered_items)

        if found != count:
            LOG.info("[failed test ctx] items=%s, expected_props=%s" % (str(
                items), props))
            self.fail("Wrong number of items found [props=%s, "
                      "expected=%s, found=%s]" % (props, count, found))

        return filtered_items


class DbTestCase(BaseTest):
    def setUp(self):
        self.db_fd, self.db_path = tempfile.mkstemp()
        session.set_defaults('sqlite:///' + self.db_path, self.db_path)
        db_api.setup_db()

    def tearDown(self):
        db_api.drop_db()
        os.close(self.db_fd)
        os.unlink(self.db_path)

    def is_db_session_open(self):
        return db_api._get_thread_local_session() is not None


class EngineTestCase(DbTestCase):
    transport = get_fake_transport()

    def __init__(self, *args, **kwargs):
        super(EngineTestCase, self).__init__(*args, **kwargs)
        self.engine = client.EngineClient(self.transport)

    @classmethod
    def mock_task_result(cls, workbook_name, execution_id,
                         task_id, state, result):
        """
        Mock the engine convey_task_results to send request directly
        to the engine instead of going through the oslo.messaging transport.
        """
        return engine.ScalableEngine.convey_task_result(
            workbook_name, execution_id, task_id, state, result)

    @classmethod
    def mock_start_workflow(cls, workbook_name, task_name, context=None):
        """
        Mock the engine start_workflow_execution to send request directly
        to the engine instead of going through the oslo.messaging transport.
        """
        return engine.ScalableEngine.start_workflow_execution(
            workbook_name, task_name, context)

    @classmethod
    def mock_get_workflow_state(cls, workbook_name, execution_id):
        """
        Mock the engine get_workflow_execution_state to send request directly
        to the engine instead of going through the oslo.messaging transport.
        """
        return engine.ScalableEngine.get_workflow_execution_state(
            workbook_name, execution_id)

    @classmethod
    def mock_run_tasks(cls, tasks):
        """
        Mock the engine _run_tasks to send requests directly to the task
        executor instead of going through the oslo.messaging transport.
        """
        executor = server.Executor(transport=cls.transport)
        for task in tasks:
            executor.handle_task({}, task=task)

    @classmethod
    def mock_handle_task(cls, cntx, **kwargs):
        """
        Mock the executor handle_task to send requests directory to the task
        executor instead of going through the oslo.messaging transport.
        """
        executor = server.Executor(transport=cls.transport)
        return executor.handle_task(cntx, **kwargs)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import eventlet

eventlet.monkey_patch()

import uuid
import mock

from oslo.config import cfg

from mistral.tests import base
from mistral.openstack.common import log as logging
from mistral.openstack.common import importutils
from mistral.engine import states
from mistral.db import api as db_api
from mistral.actions import std_actions
from mistral.engine import client as engine
from mistral.engine.scalable.executor import client as executor


# We need to make sure that all configuration properties are registered.
importutils.import_module("mistral.config")
LOG = logging.getLogger(__name__)

# Use the set_default method to set value otherwise in certain test cases
# the change in value is not permanent.
cfg.CONF.set_default('auth_enable', False, group='pecan')


WORKBOOK_NAME = 'my_workbook'
TASK_NAME = 'create-vms'

SAMPLE_WORKBOOK = {
    'id': str(uuid.uuid4()),
    'name': WORKBOOK_NAME,
    'description': 'my description',
    'definition': base.get_resource("test_rest.yaml"),
    'tags': [],
    'scope': 'public',
    'updated_at': None,
    'project_id': '123',
    'trust_id': '1234'
}

SAMPLE_EXECUTION = {
    'id': str(uuid.uuid4()),
    'workbook_name': WORKBOOK_NAME,
    'task': TASK_NAME,
    'state': states.RUNNING,
    'updated_at': None,
    'context': None
}

SAMPLE_TASK = {
    'name': TASK_NAME,
    'workbook_name': WORKBOOK_NAME,
    'action_spec': {
        'name': 'my-action',
        'class': 'std.http',
        'base-parameters': {
            'url': 'http://localhost:8989/v1/workbooks',
            'method': 'GET'
        },
        'namespace': 'MyRest'
    },
    'task_spec': {
        'action': 'MyRest.my-action',
        'name': TASK_NAME},
    'requires': {},
    'state': states.IDLE}

SAMPLE_CONTEXT = {
    'user': 'admin',
    'tenant': 'mistral'
}


class TestExecutor(base.DbTestCase):
    def __init__(self, *args, **kwargs):
        super(TestExecutor, self).__init__(*args, **kwargs)
        self.transport = base.get_fake_transport()

    @mock.patch.object(
        executor.ExecutorClient, 'handle_task',
        mock.MagicMock(side_effect=base.EngineTestCase.mock_handle_task))
    @mock.patch.object(
        std_actions.HTTPAction, 'run', mock.MagicMock(return_value={}))
    @mock.patch.object(
        engine.EngineClient, 'convey_task_result',
        mock.MagicMock(side_effect=base.EngineTestCase.mock_task_result))
    def test_handle_task(self):
        # Create a new workbook.
        workbook = db_api.workbook_create(SAMPLE_WORKBOOK)
        self.assertIsInstance(workbook, dict)

        # Create a new execution.
        execution = db_api.execution_create(SAMPLE_EXECUTION['workbook_name'],
                                            SAMPLE_EXECUTION)
        self.assertIsInstance(execution, dict)

        # Create a new task.
        SAMPLE_TASK['execution_id'] = execution['id']
        task = db_api.task_create(SAMPLE_TASK['workbook_name'],
                                  SAMPLE_TASK['execution_id'],
                                  SAMPLE_TASK)
        self.assertIsInstance(task, dict)
        self.assertIn('id', task)

        # Send the task request to the Executor.
        ex_client = executor.ExecutorClient(self.transport)
        ex_client.handle_task(SAMPLE_CONTEXT, task=task)

        # Check task execution state.
        db_task = db_api.task_get(task['workbook_name'],
                                  task['execution_id'],
                                  task['id'])
        self.assertEqual(db_task['state'], states.SUCCESS)



# -*- coding: utf-8 -*-
#
# Copyright 2014 - StackStorm, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import mock

import eventlet

from oslo.config import cfg

from mistral import exceptions as exc
from mistral.openstack.common import log as logging
from mistral.tests import base
from mistral.db import api as db_api
from mistral.engine import client
from mistral.engine.scalable import engine
from mistral.actions import std_actions
from mistral.engine import states
from mistral import dsl_parser as parser


LOG = logging.getLogger(__name__)

# Use the set_default method to set value otherwise in certain test cases
# the change in value is not permanent.
cfg.CONF.set_default('auth_enable', False, group='pecan')

WB_NAME = "my_workbook"


def _get_workbook(workbook_name):
    wb = db_api.workbook_get(workbook_name)
    return parser.get_workbook(wb["definition"])


class FailBeforeSuccessMocker(object):
    def __init__(self, fail_count):
        self._max_fail_count = fail_count
        self._call_count = 0

    def mock_partial_failure(self, *args):
        if self._call_count < self._max_fail_count:
            self._call_count += 1
            raise exc.ActionException()

        return "result"


@mock.patch.object(
    client.EngineClient, 'start_workflow_execution',
    mock.MagicMock(side_effect=base.EngineTestCase.mock_start_workflow))
@mock.patch.object(
    client.EngineClient, 'convey_task_result',
    mock.MagicMock(side_effect=base.EngineTestCase.mock_task_result))
@mock.patch.object(
    engine.ScalableEngine, '_run_tasks',
    mock.MagicMock(side_effect=base.EngineTestCase.mock_run_tasks))
@mock.patch.object(
    db_api, 'workbook_get',
    mock.MagicMock(
        return_value={
            'definition': base.get_resource('retry_task/retry_task.yaml')}))
@mock.patch.object(
    std_actions.HTTPAction, 'run',
    mock.MagicMock(return_value='result'))
class TaskRetryTest(base.EngineTestCase):

    def test_no_retry(self):
        execution = self.engine.start_workflow_execution(WB_NAME,
                                                         'retry_task', None)
        tasks = db_api.tasks_get(WB_NAME, execution['id'])

        self.engine.convey_task_result(WB_NAME, execution['id'],
                                       tasks[0]['id'], states.SUCCESS,
                                       {'output': 'result'})

        # TODO(rakhmerov): It's not stable, need to avoid race condition.
        self._assert_single_item(tasks, name='retry_task')
        self._assert_single_item(tasks, task_runtime_context=None)

    def test_retry_always_error(self):
        workbook = _get_workbook(WB_NAME)

        execution = self.engine.start_workflow_execution(WB_NAME,
                                                         'retry_task', None)
        tasks = db_api.tasks_get(WB_NAME, execution['id'])
        task_spec = workbook.tasks.get(tasks[0]['name'])
        retry_count, _, __ = task_spec.get_retry_parameters()

        for x in xrange(0, retry_count + 1):
            self.engine.convey_task_result(WB_NAME, execution['id'],
                                           tasks[0]['id'], states.ERROR,
                                           {'output': 'result'})

        # TODO(rakhmerov): It's not stable, need to avoid race condition.
        tasks = db_api.tasks_get(WB_NAME, execution['id'])

        self._assert_single_item(tasks, name='retry_task')
        self._assert_single_item(tasks, task_runtime_context={
            'retry_no': retry_count - 1})
        self._assert_single_item(tasks, state=states.ERROR)

    def test_retry_eventual_success(self):
        workbook = _get_workbook(WB_NAME)

        execution = self.engine.start_workflow_execution(WB_NAME,
                                                         'retry_task', None)
        tasks = db_api.tasks_get(WB_NAME, execution['id'])
        task_spec = workbook.tasks.get(tasks[0]['name'])
        retry_count, _, __ = task_spec.get_retry_parameters()

        for x in xrange(0, retry_count/2):
            self.engine.convey_task_result(WB_NAME, execution['id'],
                                           tasks[0]['id'], states.ERROR,
                                           {'output': 'result'})

        self.engine.convey_task_result(WB_NAME, execution['id'],
                                       tasks[0]['id'], states.SUCCESS,
                                       {'output': 'result'})

        # TODO(rakhmerov): It's not stable, need to avoid race condition.
        tasks = db_api.tasks_get(WB_NAME, execution['id'])

        self._assert_single_item(tasks, name='retry_task')
        self._assert_single_item(tasks, task_runtime_context={
            'retry_no': retry_count/2 - 1})

    def test_retry_delay(self):
        task_name = 'delay_retry_task'
        workbook = _get_workbook(WB_NAME)

        execution = self.engine.start_workflow_execution(WB_NAME,
                                                         task_name, None)

        tasks = db_api.tasks_get(WB_NAME, execution['id'])
        task_spec = workbook.tasks.get(tasks[0]['name'])
        retry_count, _, delay = task_spec.get_retry_parameters()

        for x in xrange(0, retry_count):
            self.engine.convey_task_result(WB_NAME, execution['id'],
                                           tasks[0]['id'], states.ERROR,
                                           {'output': 'result'})

            tasks = db_api.tasks_get(WB_NAME, execution['id'])

            # TODO(rakhmerov): It's not stable, need to avoid race condition.
            self._assert_single_item(tasks, name=task_name)
            self._assert_single_item(tasks, state=states.DELAYED)

            eventlet.sleep(delay * 2)

        # Convey final result outside the loop.
        self.engine.convey_task_result(WB_NAME, execution['id'],
                                       tasks[0]['id'], states.ERROR,
                                       {'output': 'result'})

        # TODO(rakhmerov): It's not stable, need to avoid race condition.
        tasks = db_api.tasks_get(WB_NAME, execution['id'])

        self._assert_single_item(tasks, name=task_name)
        self._assert_single_item(tasks, task_runtime_context={
            'retry_no': retry_count - 1})
        self._assert_single_item(tasks, state=states.ERROR)

    def test_from_no_retry_to_retry_task(self):
        task_name_1 = 'no_retry_task'
        task_name_2 = 'delay_retry_task'
        workbook = _get_workbook(WB_NAME)

        execution = self.engine.start_workflow_execution(WB_NAME,
                                                         task_name_1, None)

        tasks = db_api.tasks_get(WB_NAME, execution['id'])

        self._assert_single_item(tasks, name=task_name_1)

        self.engine.convey_task_result(WB_NAME, execution['id'],
                                       tasks[0]['id'], states.SUCCESS,
                                       {'output': 'result'})

        tasks = db_api.tasks_get(WB_NAME, execution['id'])

        self._assert_single_item(tasks, name=task_name_2)

        task_spec = workbook.tasks.get(task_name_2)
        retry_count, _, delay = task_spec.get_retry_parameters()

        for x in xrange(0, retry_count):
            self.engine.convey_task_result(WB_NAME, execution['id'],
                                           tasks[1]['id'], states.ERROR,
                                           {'output': 'result'})

            tasks = db_api.tasks_get(WB_NAME, execution['id'])

            # TODO(rakhmerov): It's not stable, need to avoid race condition.
            self._assert_single_item(tasks, name=task_name_1)
            self._assert_single_item(tasks, state=states.DELAYED)

            eventlet.sleep(delay * 2)

        # Convey final result outside the loop.
        self.engine.convey_task_result(WB_NAME, execution['id'],
                                       tasks[1]['id'], states.ERROR,
                                       {'output': 'result'})

        # TODO(rakhmerov): It's not stable, need to avoid race condition.
        tasks = db_api.tasks_get(WB_NAME, execution['id'])

        self._assert_single_item(tasks, name=task_name_2)
        self._assert_single_item(tasks, task_runtime_context={
            'retry_no': retry_count - 1})
        self._assert_single_item(tasks, state=states.ERROR)

    @mock.patch.object(std_actions.EchoAction, "run",
                       mock.MagicMock(side_effect=exc.ActionException))
    def test_sync_action_always_error(self):
        task_name_1 = 'sync_task'
        workbook = _get_workbook(WB_NAME)
        task_spec = workbook.tasks.get(task_name_1)
        retry_count, _, __ = task_spec.get_retry_parameters()

        execution = self.engine.start_workflow_execution(WB_NAME,
                                                         task_name_1, None)

        # TODO(rakhmerov): It's not stable, need to avoid race condition.
        tasks = db_api.tasks_get(WB_NAME, execution['id'])

        self._assert_single_item(tasks, name=task_name_1)
        self._assert_single_item(tasks, task_runtime_context={
            'retry_no': retry_count - 1})
        self._assert_single_item(tasks, state=states.ERROR)

    def test_sync_action_eventual_success(self):
        task_name_1 = 'sync_task'
        workbook = _get_workbook(WB_NAME)
        task_spec = workbook.tasks.get(task_name_1)
        retry_count, _, __ = task_spec.get_retry_parameters()

        # After a pre-set no of retries the mock method will return a
        # success to simulate this test-case.
        mock_functor = FailBeforeSuccessMocker(retry_count/2 + 1)

        with mock.patch.object(std_actions.EchoAction, "run",
                               side_effect=mock_functor.mock_partial_failure):
            execution = self.engine.start_workflow_execution(WB_NAME,
                                                             task_name_1, None)

            # TODO(rakhmerov): It's not stable, need to avoid race condition.
            tasks = db_api.tasks_get(WB_NAME, execution['id'])

            self._assert_single_item(tasks, name=task_name_1)
            self._assert_single_item(tasks, task_runtime_context={
                'retry_no': retry_count/2})
            self._assert_single_item(tasks, state=states.SUCCESS)



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from mistral.workbook import base
from mistral.workbook import namespaces
from mistral.workbook import workflow


class WorkbookSpec(base.BaseSpec):
    _required_keys = ['Namespaces', 'Workflow']

    def __init__(self, doc):
        super(WorkbookSpec, self).__init__(doc)

        if self.validate():
            self.namespaces =\
                namespaces.NamespaceSpecList(self._data['Namespaces'])
            self.workflow = workflow.WorkflowSpec(self._data['Workflow'])
            self.tasks = self.workflow.tasks

    def get_triggers(self):
        triggers_from_data = self._data.get("triggers", None)

        if not triggers_from_data:
            return []

        triggers = []
        for name in triggers_from_data:
            trigger_dict = {'name': name}
            trigger_dict.update(triggers_from_data[name])
            triggers.append(trigger_dict)

        return triggers

    def get_action(self, task_action_name):
        if task_action_name.find(":") == -1:
            return {}

        namespace_name = task_action_name.split(':')[0]
        action_name = task_action_name.split(':')[1]
        action = self.namespaces.get(namespace_name).actions.get(action_name)

        return action

    def get_actions(self, namespace_name):
        return self.namespaces.get(namespace_name).actions

    def get_trigger_task_name(self, trigger_name):
        trigger = self._data["triggers"].get(trigger_name)
        return trigger.get('tasks') if trigger else ""



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import mock

from mistral.tests.api import base
from mistral.db import api as db_api
from mistral.engine import client

# TODO: later we need additional tests verifying all the errors etc.

TASKS = [
    {
        'id': "1",
        'workbook_name': "my_workbook",
        'execution_id': '123',
        'name': 'my_task',
        'description': 'My cool task',
        'action': 'my_action',
        'state': 'RUNNING',
        'tags': ['deployment', 'demo']
    }
]

UPDATED_TASK = TASKS[0].copy()
UPDATED_TASK['state'] = 'STOPPED'


class TestTasksController(base.FunctionalTest):
    @mock.patch.object(db_api, "task_get",
                       mock.MagicMock(return_value=TASKS[0]))
    def test_get(self):
        resp = self.app.get('/v1/workbooks/my_workbook/executions/123/tasks/1')

        self.assertEqual(resp.status_int, 200)
        self.assertDictEqual(TASKS[0], resp.json)

    @mock.patch.object(client.EngineClient, "convey_task_result",
                       mock.MagicMock(return_value=UPDATED_TASK))
    def test_put(self):
        resp = self.app.put_json(
            '/v1/workbooks/my_workbook/executions/123/tasks/1',
            dict(state='STOPPED'))

        self.assertEqual(resp.status_int, 200)
        self.assertDictEqual(UPDATED_TASK, resp.json)

    @mock.patch.object(db_api, "tasks_get",
                       mock.MagicMock(return_value=TASKS))
    def test_get_all(self):
        resp = self.app.get('/v1/workbooks/my_workbook/executions/123/tasks')

        self.assertEqual(resp.status_int, 200)

        self.assertEqual(len(resp.json), 1)
        self.assertDictEqual(TASKS[0], resp.json['tasks'][0])



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import abc
import re
import yaql
import six

from mistral.openstack.common import log as logging

LOG = logging.getLogger(__name__)


class Evaluator(object):
    """Expression evaluator interface.

    Having this interface gives the flexibility to change the actual expression
    language used in Mistral DSL for conditions, output calculation etc.
    """

    @classmethod
    @abc.abstractmethod
    def evaluate(cls, expression, context):
        """Evaluates the expression against the given data context.

        :param expression: Expression string
        :param context: Data context
        :return: Expression result
        """
        pass

    @classmethod
    @abc.abstractmethod
    def is_expression(cls, expression):
        """
        Check expression string and decide whether it is expression or not.

        :param expression: Expression string
        :return: True if string is expression
        """
        pass


class YAQLEvaluator(Evaluator):
    @classmethod
    def evaluate(cls, expression, context):
        LOG.debug("Evaluating YAQL expression [expression='%s', context=%s]"
                  % (expression, context))

        return yaql.parse(expression).evaluate(context)

    @classmethod
    def is_expression(cls, s):
        # TODO(rakhmerov): It should be generalized since it may not be YAQL.
        return s and s.startswith('$')


class InlineYAQLEvaluator(YAQLEvaluator):
    # Put YAQL-specific regexp pattern here.
    # Use form {$.any_symbols_except'}'} to find an expression.
    find_expression_pattern = re.compile("\{\$\.*[^\}]*\}")

    @classmethod
    def evaluate(cls, expression, context):
        if super(InlineYAQLEvaluator, cls).is_expression(expression):
            return super(InlineYAQLEvaluator,
                         cls).evaluate(expression, context)

        result = expression
        found_expressions = cls.find_inline_expressions(expression)

        if found_expressions:
            for expr in found_expressions:
                trim_expr = expr.strip("{}")
                evaluated = super(InlineYAQLEvaluator,
                                  cls).evaluate(trim_expr, context)

                replacement = str(evaluated) if evaluated else expr
                result = result.replace(expr, replacement)

        return result

    @classmethod
    def is_expression(cls, s):
        return s

    @classmethod
    def find_inline_expressions(cls, s):
        return cls.find_expression_pattern.findall(s)


# TODO(rakhmerov): Make it configurable.
_EVALUATOR = InlineYAQLEvaluator


def evaluate(expression, context):
    # Check if the passed value is expression so we don't need to do this
    # every time on a caller side.
    if not isinstance(expression, six.string_types) or \
            not _EVALUATOR.is_expression(expression):
        return expression

    return _EVALUATOR.evaluate(expression, context)


def _evaluate_item(item, context):
    if isinstance(item, six.string_types):
        try:
            return evaluate(item, context)
        except AttributeError as e:
            LOG.debug("Expression %s is not evaluated, [context=%s]: %s"
                      % (item, context, e))
            return item
    else:
        return evaluate_recursively(item, context)


def evaluate_recursively(data, context):
    if not context:
        return data

    if isinstance(data, dict):
        for key in data:
            data[key] = _evaluate_item(data[key], context)
    elif isinstance(data, list):
        for index, item in enumerate(data):
            data[index] = _evaluate_item(item, context)
    elif isinstance(data, six.string_types):
        return _evaluate_item(data, context)

    return data



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from oslo import messaging
from oslo.config import cfg
from mistral.openstack.common import log as logging
from mistral.engine.scalable.executor import client
from mistral.engine import abstract_engine as abs_eng


LOG = logging.getLogger(__name__)


class ScalableEngine(abs_eng.AbstractEngine):
    @classmethod
    def _notify_task_executors(cls, tasks):
        # TODO(m4dcoder): Use a pool for transport and client
        if not cls.transport:
            cls.transport = messaging.get_transport(cfg.CONF)
        ex_client = client.ExecutorClient(cls.transport)
        for task in tasks:
            # TODO(m4dcoder): Fill request context argument with auth info
            context = {}
            ex_client.handle_task(context, task=task)
            LOG.info("Submitted task for execution: '%s'" % task)

    @classmethod
    def _run_tasks(cls, tasks):
        # TODO(rakhmerov):
        # This call outside of DB transaction creates a window
        # when the engine may crash and DB will not be consistent with
        # the task message queue state. Need to figure out the best
        # solution to recover from this situation.
        # However, making this call in DB transaction is really bad
        # since it makes transaction much longer in time and under load
        # may overload DB with open transactions.
        cls._notify_task_executors(tasks)


def get_engine():
    return ScalableEngine



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import pecan
from pecan import rest
from pecan import abort
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from mistral.openstack.common import log as logging
from mistral.api.controllers import resource
from mistral.db import api as db_api


LOG = logging.getLogger(__name__)


class Task(resource.Resource):
    """Task resource."""

    id = wtypes.text
    workbook_name = wtypes.text
    execution_id = wtypes.text
    name = wtypes.text
    description = wtypes.text
    action = wtypes.text
    state = wtypes.text
    tags = [wtypes.text]


class Tasks(resource.Resource):
    """A collection of tasks."""

    tasks = [Task]


class TasksController(rest.RestController):
    @wsme_pecan.wsexpose(Task, wtypes.text, wtypes.text, wtypes.text)
    def get(self, workbook_name, execution_id, id):
        LOG.debug("Fetch task [workbook_name=%s, execution_id=%s, id=%s]" %
                  (workbook_name, execution_id, id))

        values = db_api.task_get(workbook_name, execution_id, id)

        if not values:
            abort(404)

        return Task.from_dict(values)

    @wsme_pecan.wsexpose(Task, wtypes.text, wtypes.text, wtypes.text,
                         body=Task)
    def put(self, workbook_name, execution_id, id, task):
        LOG.debug("Update task "
                  "[workbook_name=%s, execution_id=%s, id=%s, task=%s]" %
                  (workbook_name, execution_id, id, task))

        # TODO(rakhmerov): pass task result once it's implemented
        engine = pecan.request.context['engine']
        values = engine.convey_task_result(workbook_name,
                                           execution_id,
                                           id,
                                           task.state, None)

        return Task.from_dict(values)

    @wsme_pecan.wsexpose(Tasks, wtypes.text, wtypes.text)
    def get_all(self, workbook_name, execution_id):
        LOG.debug("Fetch tasks [workbook_name=%s, execution_id=%s]" %
                  (workbook_name, execution_id))

        tasks = [Task.from_dict(values)
                 for values in db_api.tasks_get(workbook_name, execution_id)]

        return Tasks(tasks=tasks)



# -*- coding: utf-8 -*-
#
# Copyright 2014 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import paramiko

from mistral.openstack.common import log as logging


LOG = logging.getLogger(__name__)


def _read_paramimko_stream(recv_func):
    result = ''
    buf = recv_func(1024)
    while buf != '':
        result += buf
        buf = recv_func(1024)

    return result


def _connect(host, username, password):
    LOG.debug('Creating SSH connection to %s' % host)
    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(host, username=username, password=password)
    return ssh


def _cleanup(ssh):
    ssh.close()


def execute_command(cmd, host, username, password,
                    get_stderr=False, raise_when_error=True):
    ssh = _connect(host, username, password)

    LOG.debug("Executing command %s")

    try:
        chan = ssh.get_transport().open_session()
        chan.exec_command(cmd)

        # TODO (nmakhotkin): that could hang if stderr buffer overflows
        stdout = _read_paramimko_stream(chan.recv)
        stderr = _read_paramimko_stream(chan.recv_stderr)

        ret_code = chan.recv_exit_status()

        if ret_code and raise_when_error:
            raise RuntimeError("Cmd: %s\nReturn code: %s\nstdout: %s"
                               % (cmd, ret_code, stdout))
        if get_stderr:
            return ret_code, stdout, stderr
        else:
            return ret_code, stdout
    finally:
        _cleanup(ssh)



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import pkg_resources as pkg

from mistral.db import api as db_api
from mistral.tests import base
from mistral import version
from mistral.services import scheduler


class TriggersTest(base.DbTestCase):
    def setUp(self):
        super(TriggersTest, self).setUp()

        self.doc = open(pkg.resource_filename(
            version.version_info.package,
            "tests/resources/test_rest.yaml")).read()

    def test_create_associated_triggers(self):
        workbook = {
            'name': 'my_workbook',
            'definition': self.doc
        }

        scheduler.create_associated_triggers(workbook)

        triggers = db_api.triggers_get(workbook_name='my_workbook')

        self.assertEqual(triggers[0]['name'], 'create-vms')
        self.assertEqual(triggers[0]['pattern'], '* * * * *')
        self.assertEqual(triggers[0]['workbook_name'], 'my_workbook')



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""
Configuration options registration and useful routines.
"""

from oslo.config import cfg
from keystoneclient.middleware import auth_token

from mistral.openstack.common import log
from mistral import version

api_opts = [
    cfg.StrOpt('host', default='0.0.0.0', help='Mistral API server host'),
    cfg.IntOpt('port', default=8989, help='Mistral API server port')
]

engine_opts = [
    cfg.StrOpt('engine', default='mistral.engine.scalable.engine',
               help='Mistral engine class'),
    cfg.StrOpt('host', default='0.0.0.0',
               help='Name of the engine node. This can be an opaque '
                    'identifier. It is not necessarily a hostname, '
                    'FQDN, or IP address.'),
    cfg.StrOpt('topic', default='engine',
               help='The message topic that the engine listens on.'),
    cfg.StrOpt('version', default='1.0',
               help='The version of the engine.')
]

pecan_opts = [
    cfg.StrOpt('root', default='mistral.api.controllers.root.RootController',
               help='Pecan root controller'),
    cfg.ListOpt('modules', default=["mistral.api"]),
    cfg.BoolOpt('debug', default=False),
    cfg.BoolOpt('auth_enable', default=True)
]

db_opts = [
    # TODO: add DB properties.
]

rabbit_opts = [
    cfg.StrOpt('rabbit_host', default='0.0.0.0',
               help='RabbitMQ server host name'),
    cfg.IntOpt('rabbit_port', default=5672, help='RabbitMQ server port'),
    cfg.StrOpt('rabbit_virtual_host', default='/',
               help='RabbitMQ server virtual host name'),
    cfg.StrOpt('rabbit_task_queue', default='tasks',
               help='RabbitMQ tasks queue name'),
    cfg.StrOpt('rabbit_user', default='guest', help='RabbitMQ user'),
    cfg.StrOpt('rabbit_password', default='guest', help='RabbitMQ password')
]

use_debugger = cfg.BoolOpt(
    "use-debugger",
    default=False,
    help='Enables debugger. Note that using this option changes how the '
    'eventlet library is used to support async IO. This could result '
    'in failures that do not occur under normal operation. '
    'Use at your own risk.'
)

executor_opts = [
    cfg.StrOpt('host', default='0.0.0.0',
               help='Name of the executor node. This can be an opaque '
                    'identifier. It is not necessarily a hostname, '
                    'FQDN, or IP address.'),
    cfg.StrOpt('topic', default='executor',
               help='The message topic that the executor listens on.'),
    cfg.StrOpt('version', default='1.0',
               help='The version of the executor.')
]

launch_opt = cfg.StrOpt(
    'server',
    default='all',
    choices=('all', 'api', 'engine', 'executor'),
    help='Specifies which mistral server to start by the launch script.'
)


wf_trace_log_name_opt = cfg.StrOpt('workflow_trace_log_name',
                                   default='workflow_trace',
                                   help='Logger name for pretty '
                                   'workflow trace output.')

CONF = cfg.CONF

CONF.register_opts(api_opts, group='api')
CONF.register_opts(engine_opts, group='engine')
CONF.register_opts(pecan_opts, group='pecan')
CONF.register_opts(auth_token.opts, group='keystone')
CONF.register_opts(db_opts, group='database')
CONF.register_opts(rabbit_opts, group='rabbit')
CONF.register_opts(executor_opts, group='executor')
CONF.register_opt(wf_trace_log_name_opt)

CONF.register_cli_opt(use_debugger)
CONF.register_cli_opt(launch_opt)

CONF.import_opt('verbose', 'mistral.openstack.common.log')
CONF.import_opt('debug', 'mistral.openstack.common.log')
CONF.import_opt('log_dir', 'mistral.openstack.common.log')
CONF.import_opt('log_file', 'mistral.openstack.common.log')
CONF.import_opt('log_config_append', 'mistral.openstack.common.log')
CONF.import_opt('log_format', 'mistral.openstack.common.log')
CONF.import_opt('log_date_format', 'mistral.openstack.common.log')
CONF.import_opt('use_syslog', 'mistral.openstack.common.log')
CONF.import_opt('syslog_log_facility', 'mistral.openstack.common.log')

cfg.set_defaults(log.log_opts,
                 default_log_levels=['sqlalchemy=WARN',
                                     'eventlet.wsgi.server=WARN'])


def parse_args(args=None, usage=None, default_config_files=None):
    CONF(args=args,
         project='mistral',
         version=version,
         usage=usage,
         default_config_files=default_config_files)



# Copyright 2013 OpenStack Foundation
# Copyright 2013 Spanish National Research Council.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# E0202: An attribute inherited from %s hide this method
# pylint: disable=E0202

import abc
import argparse
import logging
import os

import six
from stevedore import extension

from mistral.openstack.common.apiclient import exceptions


logger = logging.getLogger(__name__)


_discovered_plugins = {}


def discover_auth_systems():
    """Discover the available auth-systems.

    This won't take into account the old style auth-systems.
    """
    global _discovered_plugins
    _discovered_plugins = {}

    def add_plugin(ext):
        _discovered_plugins[ext.name] = ext.plugin

    ep_namespace = "mistral.openstack.common.apiclient.auth"
    mgr = extension.ExtensionManager(ep_namespace)
    mgr.map(add_plugin)


def load_auth_system_opts(parser):
    """Load options needed by the available auth-systems into a parser.

    This function will try to populate the parser with options from the
    available plugins.
    """
    group = parser.add_argument_group("Common auth options")
    BaseAuthPlugin.add_common_opts(group)
    for name, auth_plugin in _discovered_plugins.iteritems():
        group = parser.add_argument_group(
            "Auth-system '%s' options" % name,
            conflict_handler="resolve")
        auth_plugin.add_opts(group)


def load_plugin(auth_system):
    try:
        plugin_class = _discovered_plugins[auth_system]
    except KeyError:
        raise exceptions.AuthSystemNotFound(auth_system)
    return plugin_class(auth_system=auth_system)


def load_plugin_from_args(args):
    """Load required plugin and populate it with options.

    Try to guess auth system if it is not specified. Systems are tried in
    alphabetical order.

    :type args: argparse.Namespace
    :raises: AuthorizationFailure
    """
    auth_system = args.os_auth_system
    if auth_system:
        plugin = load_plugin(auth_system)
        plugin.parse_opts(args)
        plugin.sufficient_options()
        return plugin

    for plugin_auth_system in sorted(_discovered_plugins.iterkeys()):
        plugin_class = _discovered_plugins[plugin_auth_system]
        plugin = plugin_class()
        plugin.parse_opts(args)
        try:
            plugin.sufficient_options()
        except exceptions.AuthPluginOptionsMissing:
            continue
        return plugin
    raise exceptions.AuthPluginOptionsMissing(["auth_system"])


@six.add_metaclass(abc.ABCMeta)
class BaseAuthPlugin(object):
    """Base class for authentication plugins.

    An authentication plugin needs to override at least the authenticate
    method to be a valid plugin.
    """

    auth_system = None
    opt_names = []
    common_opt_names = [
        "auth_system",
        "username",
        "password",
        "tenant_name",
        "token",
        "auth_url",
    ]

    def __init__(self, auth_system=None, **kwargs):
        self.auth_system = auth_system or self.auth_system
        self.opts = dict((name, kwargs.get(name))
                         for name in self.opt_names)

    @staticmethod
    def _parser_add_opt(parser, opt):
        """Add an option to parser in two variants.

        :param opt: option name (with underscores)
        """
        dashed_opt = opt.replace("_", "-")
        env_var = "OS_%s" % opt.upper()
        arg_default = os.environ.get(env_var, "")
        arg_help = "Defaults to env[%s]." % env_var
        parser.add_argument(
            "--os-%s" % dashed_opt,
            metavar="<%s>" % dashed_opt,
            default=arg_default,
            help=arg_help)
        parser.add_argument(
            "--os_%s" % opt,
            metavar="<%s>" % dashed_opt,
            help=argparse.SUPPRESS)

    @classmethod
    def add_opts(cls, parser):
        """Populate the parser with the options for this plugin.
        """
        for opt in cls.opt_names:
            # use `BaseAuthPlugin.common_opt_names` since it is never
            # changed in child classes
            if opt not in BaseAuthPlugin.common_opt_names:
                cls._parser_add_opt(parser, opt)

    @classmethod
    def add_common_opts(cls, parser):
        """Add options that are common for several plugins.
        """
        for opt in cls.common_opt_names:
            cls._parser_add_opt(parser, opt)

    @staticmethod
    def get_opt(opt_name, args):
        """Return option name and value.

        :param opt_name: name of the option, e.g., "username"
        :param args: parsed arguments
        """
        return (opt_name, getattr(args, "os_%s" % opt_name, None))

    def parse_opts(self, args):
        """Parse the actual auth-system options if any.

        This method is expected to populate the attribute `self.opts` with a
        dict containing the options and values needed to make authentication.
        """
        self.opts.update(dict(self.get_opt(opt_name, args)
                              for opt_name in self.opt_names))

    def authenticate(self, http_client):
        """Authenticate using plugin defined method.

        The method usually analyses `self.opts` and performs
        a request to authentication server.

        :param http_client: client object that needs authentication
        :type http_client: HTTPClient
        :raises: AuthorizationFailure
        """
        self.sufficient_options()
        self._do_authenticate(http_client)

    @abc.abstractmethod
    def _do_authenticate(self, http_client):
        """Protected method for authentication.
        """

    def sufficient_options(self):
        """Check if all required options are present.

        :raises: AuthPluginOptionsMissing
        """
        missing = [opt
                   for opt in self.opt_names
                   if not self.opts.get(opt)]
        if missing:
            raise exceptions.AuthPluginOptionsMissing(missing)

    @abc.abstractmethod
    def token_and_endpoint(self, endpoint_type, service_type):
        """Return token and endpoint.

        :param service_type: Service type of the endpoint
        :type service_type: string
        :param endpoint_type: Type of endpoint.
                              Possible values: public or publicURL,
                                  internal or internalURL,
                                  admin or adminURL
        :type endpoint_type: string
        :returns: tuple of token and endpoint strings
        :raises: EndpointException
        """



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import sys
import eventlet

eventlet.monkey_patch(
    os=True,
    select=True,
    socket=True,
    thread=False if '--use-debugger' in sys.argv else True,
    time=True)

import os

# If ../mistral/__init__.py exists, add ../ to Python search path, so that
# it will override what happens to be installed in /usr/(local/)lib/python...
POSSIBLE_TOPDIR = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
                                   os.pardir,
                                   os.pardir))
if os.path.exists(os.path.join(POSSIBLE_TOPDIR, 'mistral', '__init__.py')):
    sys.path.insert(0, POSSIBLE_TOPDIR)

from oslo import messaging
from oslo.config import cfg

from mistral import config
from mistral import engine
from mistral.engine.scalable.executor import server
from mistral.api import app
from wsgiref import simple_server
from mistral.openstack.common import log as logging


LOG = logging.getLogger(__name__)


def launch_executor(transport):
    target = messaging.Target(topic=cfg.CONF.executor.topic,
                              server=cfg.CONF.executor.host)
    endpoints = [server.Executor(transport)]
    ex_server = messaging.get_rpc_server(
        transport, target, endpoints, executor='eventlet')
    ex_server.start()
    ex_server.wait()


def launch_engine(transport):
    target = messaging.Target(topic=cfg.CONF.engine.topic,
                              server=cfg.CONF.engine.host)
    endpoints = [engine.Engine(transport)]
    en_server = messaging.get_rpc_server(
        transport, target, endpoints, executor='eventlet')
    en_server.start()
    en_server.wait()


def launch_api(transport):
    host = cfg.CONF.api.host
    port = cfg.CONF.api.port
    server = simple_server.make_server(host, port,
                                       app.setup_app(transport=transport))
    LOG.info("Mistral API is serving on http://%s:%s (PID=%s)" %
             (host, port, os.getpid()))
    server.serve_forever()


def launch_all(transport):
    # Launch the servers on different threads.
    t1 = eventlet.spawn(launch_executor, transport)
    t2 = eventlet.spawn(launch_engine, transport)
    t3 = eventlet.spawn(launch_api, transport)
    t1.wait() and t2.wait() and t3.wait()


def main():
    try:
        config.parse_args()
        logging.setup('Mistral')

        # Map cli options to appropriate functions. The cli options are
        # registered in mistral's config.py.
        launch_options = {
            'all': launch_all,
            'api': launch_api,
            'engine': launch_engine,
            'executor': launch_executor
        }

        # Please refer to the oslo.messaging documentation for transport
        # configuration. The default transport for oslo.messaging is rabbitMQ.
        # The available transport drivers are listed under oslo.messaging at
        # ./oslo/messaging/rpc/_drivers.  The drivers are prefixed with "impl".
        # The transport driver is specified using the rpc_backend option in the
        # default section of the oslo configuration file. The expected value
        # for rpc_backend is the last part of the driver name. For example,
        # the driver for rabbit is impl_rabbit and for the fake driver is
        # impl_fake. The rpc_backend value for these are "rabbit" and "fake"
        # respectively. There are additional options such as ssl and credential
        # that can be specified depending on the driver.  Please refer to the
        # driver implementation for those additional options.
        transport = messaging.get_transport(cfg.CONF)

        # Launch server(s).
        launch_options[cfg.CONF.server](transport)

    except RuntimeError, e:
        sys.stderr.write("ERROR: %s\n" % e)
        sys.exit(1)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import copy
import mock

from oslo.config import cfg

from mistral.openstack.common import log as logging
from mistral.tests import base
from mistral.db import api as db_api
from mistral.engine.scalable import engine
from mistral.actions import std_actions
from mistral.engine import states
from mistral.engine import client
from mistral.utils.openstack import keystone


# TODO(rakhmerov): add more tests


LOG = logging.getLogger(__name__)

TOKEN = "123ab"
USER_ID = "321ba"

CONTEXT = {
    'person': {
        'first_name': 'John',
        'last_name': 'Doe',
        'address': {
            'street': '124352 Broadway Street',
            'city': 'Gloomington',
            'country': 'USA'
        }
    }
}

# Use the set_default method to set value otherwise in certain test cases
# the change in value is not permanent.
cfg.CONF.set_default('auth_enable', False, group='pecan')


def create_workbook(definition_path):
    return db_api.workbook_create({
        'name': 'my_workbook',
        'definition': base.get_resource(definition_path)
    })


@mock.patch.object(
    client.EngineClient, 'start_workflow_execution',
    mock.MagicMock(side_effect=base.EngineTestCase.mock_start_workflow))
@mock.patch.object(
    client.EngineClient, 'convey_task_result',
    mock.MagicMock(side_effect=base.EngineTestCase.mock_task_result))
@mock.patch.object(
    engine.ScalableEngine, '_run_tasks',
    mock.MagicMock(side_effect=base.EngineTestCase.mock_run_tasks))
class DataFlowTest(base.EngineTestCase):
    def _check_in_context_execution(self, task):
        self.assertIn('__execution', task['in_context'])

        exec_dict = task['in_context']['__execution']

        self.assertEqual('my_workbook', exec_dict['workbook_name'])
        self.assertEqual(task['execution_id'], exec_dict['id'])
        self.assertIn('task', exec_dict)

    def test_two_dependent_tasks(self):
        CTX = copy.copy(CONTEXT)

        wb = create_workbook('data_flow/two_dependent_tasks.yaml')

        execution = self.engine.start_workflow_execution(wb['name'],
                                                         'build_greeting',
                                                         CTX)

        # We have to reread execution to get its latest version.
        execution = db_api.execution_get(execution['workbook_name'],
                                         execution['id'])

        self.assertEqual(states.SUCCESS, execution['state'])
        self.assertDictEqual(CTX, execution['context'])

        tasks = db_api.tasks_get(wb['name'], execution['id'])

        self.assertEqual(2, len(tasks))

        build_full_name_task = \
            self._assert_single_item(tasks, name='build_full_name')
        build_greeting_task = \
            self._assert_single_item(tasks, name='build_greeting')

        # Check the first task.
        self.assertEqual(states.SUCCESS, build_full_name_task['state'])
        self._check_in_context_execution(build_full_name_task)
        del build_full_name_task['in_context']['__execution']
        self.assertDictEqual(CTX, build_full_name_task['in_context'])
        self.assertDictEqual({'first_name': 'John', 'last_name': 'Doe'},
                             build_full_name_task['parameters'])
        self.assertDictEqual(
            {
                'f_name': 'John Doe',
                'task': {
                    'build_full_name': {
                        'full_name': 'John Doe'
                    }
                }
            },
            build_full_name_task['output'])

        # Check the second task.
        in_context = CTX
        in_context['f_name'] = 'John Doe'

        self.assertEqual(states.SUCCESS, build_greeting_task['state'])
        self.assertEqual('John Doe',
                         build_greeting_task['in_context']['f_name'])
        self.assertDictEqual({'full_name': 'John Doe'},
                             build_greeting_task['parameters'])
        self.assertDictEqual(
            {
                'task': {
                    'build_greeting': {
                        'greeting': 'Hello, John Doe!',
                    }
                }
            },
            build_greeting_task['output'])

        del build_greeting_task['in_context']['task']

        self._check_in_context_execution(build_greeting_task)
        del build_greeting_task['in_context']['__execution']
        self.assertDictEqual(CTX, build_greeting_task['in_context'])

    def test_task_with_two_dependencies(self):
        CTX = copy.copy(CONTEXT)

        wb = create_workbook('data_flow/task_with_two_dependencies.yaml')

        execution = self.engine.start_workflow_execution(wb['name'],
                                                         'send_greeting',
                                                         CTX)

        # We have to reread execution to get its latest version.
        execution = db_api.execution_get(execution['workbook_name'],
                                         execution['id'])

        self.assertEqual(states.SUCCESS, execution['state'])
        self.assertDictEqual(CTX, execution['context'])

        tasks = db_api.tasks_get(wb['name'], execution['id'])

        self.assertEqual(3, len(tasks))

        build_full_name_task = \
            self._assert_single_item(tasks, name='build_full_name')
        build_greeting_task = \
            self._assert_single_item(tasks, name='build_greeting')
        send_greeting_task = \
            self._assert_single_item(tasks, name='send_greeting')

        # Check the first task.
        self.assertEqual(states.SUCCESS, build_full_name_task['state'])
        self._check_in_context_execution(build_full_name_task)
        del build_full_name_task['in_context']['__execution']
        self.assertDictEqual(CTX, build_full_name_task['in_context'])
        self.assertDictEqual({'first_name': 'John', 'last_name': 'Doe'},
                             build_full_name_task['parameters'])
        self.assertDictEqual(
            {
                'f_name': 'John Doe',
                'task': {
                    'build_full_name': {
                        'full_name': 'John Doe',
                    }
                }
            },
            build_full_name_task['output'])

        # Check the second task.
        in_context = CTX
        in_context['f_name'] = 'John Doe'

        self.assertEqual(states.SUCCESS, build_greeting_task['state'])
        self.assertEqual('John Doe',
                         build_greeting_task['in_context']['f_name'])
        self.assertDictEqual({}, build_greeting_task['parameters'])
        self.assertDictEqual(
            {
                'greet_msg': 'Cheers!',
                'task': {
                    'build_greeting': {
                        'greeting': 'Cheers!'
                    }
                }
            },
            build_greeting_task['output'])

        del build_greeting_task['in_context']['task']

        self._check_in_context_execution(build_greeting_task)
        del build_greeting_task['in_context']['__execution']
        self.assertDictEqual(CTX, build_greeting_task['in_context'])

        # Check the third task.
        in_context = CTX
        in_context['f_name'] = 'John Doe'
        in_context['greet_msg'] = 'Cheers!'
        in_context['task'] = {
            'build_greeting': {
                'greeting': 'Cheers!'
            }
        }

        self.assertEqual(states.SUCCESS, send_greeting_task['state'])
        self._check_in_context_execution(send_greeting_task)
        del send_greeting_task['in_context']['__execution']
        self.assertDictEqual(in_context, send_greeting_task['in_context'])
        self.assertDictEqual({'f_name': 'John Doe', 'greet_msg': 'Cheers!'},
                             send_greeting_task['parameters'])
        self.assertDictEqual(
            {
                'task': {
                    'send_greeting': {
                        'greeting_sent': True
                    }
                }
            },
            send_greeting_task['output'])

    def test_two_subsequent_tasks(self):
        CTX = copy.copy(CONTEXT)

        wb = create_workbook('data_flow/two_subsequent_tasks.yaml')

        execution = self.engine.start_workflow_execution(wb['name'],
                                                         'build_full_name',
                                                         CTX)

        # We have to reread execution to get its latest version.
        execution = db_api.execution_get(execution['workbook_name'],
                                         execution['id'])

        self.assertEqual(states.SUCCESS, execution['state'])
        self.assertDictEqual(CTX, execution['context'])

        tasks = db_api.tasks_get(wb['name'], execution['id'])

        self.assertEqual(2, len(tasks))

        build_full_name_task = \
            self._assert_single_item(tasks, name='build_full_name')
        build_greeting_task = \
            self._assert_single_item(tasks, name='build_greeting')

        # Check the first task.
        self.assertEqual(states.SUCCESS, build_full_name_task['state'])
        self._check_in_context_execution(build_full_name_task)
        del build_full_name_task['in_context']['__execution']
        self.assertDictEqual(CTX, build_full_name_task['in_context'])
        self.assertDictEqual({'first_name': 'John', 'last_name': 'Doe'},
                             build_full_name_task['parameters'])
        self.assertDictEqual(
            {
                'f_name': 'John Doe',
                'task': {
                    'build_full_name': {
                        'full_name': 'John Doe'
                    }
                }
            },
            build_full_name_task['output'])

        # Check the second task.
        in_context = CTX
        in_context['f_name'] = 'John Doe'

        self.assertEqual(states.SUCCESS, build_greeting_task['state'])
        self.assertEqual('John Doe',
                         build_greeting_task['in_context']['f_name'])
        self.assertDictEqual({'full_name': 'John Doe'},
                             build_greeting_task['parameters'])
        self.assertDictEqual(
            {
                'greet_msg': 'Hello, John Doe!',
                'task': {
                    'build_greeting': {
                        'greeting': 'Hello, John Doe!',
                    }
                }
            },
            build_greeting_task['output'])

        del build_greeting_task['in_context']['task']

        self._check_in_context_execution(build_greeting_task)
        del build_greeting_task['in_context']['__execution']
        self.assertDictEqual(CTX, build_greeting_task['in_context'])

    def test_three_subsequent_tasks(self):
        CTX = copy.copy(CONTEXT)

        wb = create_workbook('data_flow/three_subsequent_tasks.yaml')

        execution = self.engine.start_workflow_execution(wb['name'],
                                                         'build_full_name',
                                                         CTX)

        # We have to reread execution to get its latest version.
        execution = db_api.execution_get(execution['workbook_name'],
                                         execution['id'])

        self.assertEqual(states.SUCCESS, execution['state'])
        self.assertDictEqual(CTX, execution['context'])

        tasks = db_api.tasks_get(wb['name'], execution['id'])

        self.assertEqual(3, len(tasks))

        build_full_name_task = \
            self._assert_single_item(tasks, name='build_full_name')
        build_greeting_task = \
            self._assert_single_item(tasks, name='build_greeting')
        send_greeting_task = \
            self._assert_single_item(tasks, name='send_greeting')

        # Check the first task.
        self.assertEqual(states.SUCCESS, build_full_name_task['state'])
        self._check_in_context_execution(build_full_name_task)
        del build_full_name_task['in_context']['__execution']
        self.assertDictEqual(CTX, build_full_name_task['in_context'])
        self.assertDictEqual({'first_name': 'John', 'last_name': 'Doe'},
                             build_full_name_task['parameters'])
        self.assertDictEqual(
            {
                'f_name': 'John Doe',
                'task': {
                    'build_full_name': {
                        'full_name': 'John Doe'
                    }
                }
            },
            build_full_name_task['output'])

        # Check the second task.
        in_context = CTX
        in_context['f_name'] = 'John Doe'

        self.assertEqual(states.SUCCESS, build_greeting_task['state'])
        self.assertEqual('John Doe',
                         build_greeting_task['in_context']['f_name'])
        self.assertDictEqual({'full_name': 'John Doe'},
                             build_greeting_task['parameters'])
        self.assertDictEqual(
            {
                'greet_msg': 'Hello, John Doe!',
                'task': {
                    'build_greeting': {
                        'greeting': 'Hello, John Doe!',
                    }
                }
            },
            build_greeting_task['output'])

        del build_greeting_task['in_context']['task']

        self._check_in_context_execution(build_greeting_task)
        del build_greeting_task['in_context']['__execution']
        self.assertDictEqual(CTX, build_greeting_task['in_context'])

        # Check the third task.
        in_context = CTX
        in_context['f_name'] = 'John Doe'
        in_context['greet_msg'] = 'Hello, John Doe!'

        self.assertEqual(states.SUCCESS, send_greeting_task['state'])
        self.assertEqual('John Doe',
                         send_greeting_task['in_context']['f_name'])
        self.assertEqual('Hello, John Doe!',
                         send_greeting_task['in_context']['greet_msg'])
        self.assertDictEqual({'greeting': 'Hello, John Doe!'},
                             send_greeting_task['parameters'])
        self.assertDictEqual(
            {
                'sent': True,
                'task': {
                    'send_greeting': {
                        'greeting_sent': True,
                    }
                }
            },
            send_greeting_task['output'])

        del send_greeting_task['in_context']['task']

        self._check_in_context_execution(send_greeting_task)
        del send_greeting_task['in_context']['__execution']
        self.assertDictEqual(CTX, send_greeting_task['in_context'])

    @mock.patch.object(
        std_actions.HTTPAction, 'run',
        mock.MagicMock(return_value={'state': states.RUNNING}))
    @mock.patch.object(
        keystone, 'client_for_trusts',
        mock.Mock(return_value=mock.MagicMock(user_id=USER_ID,
                                              auth_token=TOKEN)))
    def test_add_token_to_context(self):
        task_name = "create-vms"

        cfg.CONF.pecan.auth_enable = True
        try:
            workbook = create_workbook("test_rest.yaml")
            db_api.workbook_update(workbook['name'], {'trust_id': '123'})

            execution = self.engine.start_workflow_execution(workbook['name'],
                                                             task_name, {})
            tasks = db_api.tasks_get(workbook['name'], execution['id'])

            task = self._assert_single_item(tasks, name=task_name)

            context = task['in_context']

            self.assertIn("auth_token", context)
            self.assertEqual(TOKEN, context['auth_token'])
            self.assertEqual(USER_ID, context["user_id"])

            self.engine.convey_task_result(workbook['name'], execution['id'],
                                           task['id'], states.SUCCESS, {})

            execution = db_api.execution_get(workbook['name'], execution['id'])

            self.assertEqual(states.SUCCESS, execution['state'])
        finally:
            cfg.CONF.pecan.auth_enable = False



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from mistral.openstack.common.db import api as db_api
from mistral.openstack.common import log as logging

# Workbooks

_BACKEND_MAPPING = {
    'sqlalchemy': 'mistral.db.sqlalchemy.api',
}

IMPL = db_api.DBAPI(backend_mapping=_BACKEND_MAPPING)
LOG = logging.getLogger(__name__)


def setup_db():
    IMPL.setup_db()


def drop_db():
    IMPL.drop_db()


# Transaction control.


def start_tx():
    IMPL.start_tx()


def commit_tx():
    IMPL.commit_tx()


def rollback_tx():
    IMPL.rollback_tx()


def end_tx():
    IMPL.end_tx()


# Workbook


def workbook_get(name):
    return IMPL.workbook_get(name)


def workbook_create(values):
    return IMPL.workbook_create(values)


def workbook_update(name, values):
    return IMPL.workbook_update(name, values)


def workbook_delete(name):
    IMPL.workbook_delete(name)


def workbooks_get():
    return IMPL.workbooks_get_all()


def workbook_definition_get(workbook_name):
    return IMPL.workbook_get(workbook_name)['definition']


def workbook_definition_put(workbook_name, text):
    return IMPL.workbook_update(workbook_name, {'definition': text})


# Executions


def execution_get(workbook_name, id):
    return IMPL.execution_get(workbook_name, id)


def execution_create(workbook_name, values):
    return IMPL.execution_create(workbook_name, values)


def execution_update(workbook_name, id, values):
    return IMPL.execution_update(workbook_name, id, values)


def execution_delete(workbook_name, id):
    return IMPL.execution_delete(workbook_name, id)


def executions_get(workbook_name):
    return IMPL.executions_get_all(workbook_name=workbook_name)


# Tasks

def task_get(workbook_name, execution_id, id):
    return IMPL.task_get(workbook_name, execution_id, id)


def task_create(workbook_name, execution_id, values):
    return IMPL.task_create(workbook_name, execution_id, values)


def task_update(workbook_name, execution_id, id, values):
    return IMPL.task_update(workbook_name, execution_id, id, values)


def task_delete(workbook_name, execution_id, id):
    return IMPL.task_delete(workbook_name, execution_id, id)


def tasks_get(workbook_name, execution_id):
    return IMPL.tasks_get_all(workbook_name=workbook_name,
                              execution_id=execution_id)


# Listeners


def listener_get(workbook_name, id):
    return {}


def listener_create(workbook_name, values):
    values['id'] = 1

    return values


def listener_update(workbook_name, id, values):
    return values


def listener_delete(workbook_name, id):
    pass


def listeners_get(workbook_name):
    return [{}]


# Triggers

def trigger_create(values):
    return IMPL.trigger_create(values)


def triggers_get(**kwargs):
    return IMPL.triggers_get_all(**kwargs)


def trigger_update(trigger_id, values):
    return IMPL.trigger_update(trigger_id, values)


def get_next_triggers(time):
    return IMPL.get_next_triggers(time)



# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from oslo import messaging
from oslo.config import cfg

from mistral.openstack.common import log as logging


LOG = logging.getLogger(__name__)


class EngineClient(object):
    """
    RPC client for the Engine.
    """

    def __init__(self, transport):
        """Construct an RPC client for the Engine.

        :param transport: a messaging transport handle
        :type transport: Transport
        """
        target = messaging.Target(topic=cfg.CONF.engine.topic)
        self._client = messaging.RPCClient(transport, target)

    def start_workflow_execution(self, workbook_name, task_name, context=None):
        """Starts a workflow execution based on the specified workbook name
        and target task.

        :param workbook_name: Workbook name
        :param task_name: Target task name
        :param context: Execution context which defines a workflow input
        :return: Workflow execution.
        """
        # TODO(m4dcoder): refactor auth context
        cntx = {}
        kwargs = {'workbook_name': workbook_name,
                  'task_name': task_name,
                  'context': context}
        return self._client.call(cntx, 'start_workflow_execution', **kwargs)

    def stop_workflow_execution(self, workbook_name, execution_id):
        """Stops the workflow execution with the given id.

        :param workbook_name: Workbook name.
        :param execution_id: Workflow execution id.
        :return: Workflow execution.
        """
        # TODO(m4dcoder): refactor auth context
        cntx = {}
        kwargs = {'workbook_name': workbook_name,
                  'execution_id': execution_id}
        return self._client.call(cntx, 'stop_workflow_execution', **kwargs)

    def convey_task_result(self, workbook_name, execution_id,
                           task_id, state, result):
        """Conveys task result to Mistral Engine.

        This method should be used by clients of Mistral Engine to update
        state of a task once task action has been performed. One of the
        clients of this method is Mistral REST API server that receives
        task result from the outside action handlers.

        Note: calling this method serves an event notifying Mistral that
        it possibly needs to move the workflow on, i.e. run other workflow
        tasks for which all dependencies are satisfied.

        :param workbook_name: Workbook name.
        :param execution_id: Workflow execution id.
        :param task_id: Task id.
        :param state: New task state.
        :param result: Task result data.
        :return: Task.
        """
        # TODO(m4dcoder): refactor auth context
        cntx = {}
        kwargs = {'workbook_name': workbook_name,
                  'execution_id': execution_id,
                  'task_id': task_id,
                  'state': state,
                  'result': result}
        return self._client.call(cntx, 'convey_task_result', **kwargs)

    def get_workflow_execution_state(self, workbook_name, execution_id):
        """Gets the workflow execution state.

        :param workbook_name: Workbook name.
        :param execution_id: Workflow execution id.
        :return: Current workflow state.
        """
        # TODO(m4dcoder): refactor auth context
        cntx = {}
        kwargs = {'workbook_name': workbook_name,
                  'execution_id': execution_id}
        return self._client.call(
            cntx, 'get_workflow_execution_state', **kwargs)

    def get_task_state(self, workbook_name, execution_id, task_id):
        """Gets task state.

        :param workbook_name: Workbook name.
        :param execution_id: Workflow execution id.
        :param task_id: Task id.
        :return: Current task state.
        """
        # TODO(m4dcoder): refactor auth context
        cntx = {}
        kwargs = {'workbook_name': workbook_name,
                  'executioin_id': execution_id,
                  'task_id': task_id}
        return self._client.call(cntx, 'get_task_state', **kwargs)



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from mistral import exceptions


class BaseSpec(object):
    _required_keys = []

    def __init__(self, data):
        self._data = data

    def validate(self):
        if not all(k in self._data for k in self._required_keys):
            message = ("Wrong model definition for: %s. It should contain"
                       " required keys: %s" % (self.__class__.__name__,
                                               self._required_keys))
            raise exceptions.InvalidModelException(message)
        return True

    def to_dict(self):
        return self._data


class BaseSpecList(object):
    item_class = None

    def __init__(self, data):
        self.items = {}

        for k, v in data.items():
            v['name'] = k
            self.items[k] = self.item_class(v)

        for name in self:
            self.get(name).validate()

    def __iter__(self):
        return iter(self.items)

    def __getitem__(self, name):
        return self.items.get(name)

    def __len__(self):
        return len(self.items)

    def get(self, name):
        return self.__getitem__(name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import time

from mistralclient.api import client as cl

client = cl.Client(project_name="mistral",
                   mistral_url="http://localhost:8989/v1",
                   username="admin",
                   api_key="secrete",
                   auth_url="http://localhost:5000/v3")

WB_NAME = "my_workbook"
TASK = "my_task"
wb_list = client.workbooks.list()

wb = None
for item in wb_list:
    if item.name == WB_NAME:
        wb = item
        break

if not wb:
    wb = client.workbooks.create(WB_NAME,
                                 description="My test workbook",
                                 tags=["test"])

print "Created workbook: %s" % wb

with open("scripts/test.yaml") as definition_file:
    definition = definition_file.read()

client.workbooks.upload_definition(WB_NAME, definition)

print "\nUploaded workbook:\n\"\n%s\"\n" %\
      client.workbooks.get_definition(WB_NAME)

execution = client.executions.create(WB_NAME, TASK)

print "execution: %s" % execution

# wait until task is complete
for i in range(0, 20):
    execution = client.executions.get(WB_NAME, execution.id)
    print "execution: %s" % execution
    if execution.state == 'SUCCESS':
        break
    time.sleep(1)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2010 OpenStack Foundation
# Copyright 2013 IBM Corp.
# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import ConfigParser
import os
import sys

import install_venv_common as install_venv  # flake8: noqa


def print_help(project, venv, root):
    help = """
    %(project)s development environment setup is complete.

    %(project)s development uses virtualenv to track and manage Python
    dependencies while in development and testing.

    To activate the %(project)s virtualenv for the extent of your current
    shell session you can run:

    $ source %(venv)s/bin/activate

    Or, if you prefer, you can run commands in the virtualenv on a case by
    case basis by running:

    $ %(root)s/tools/with_venv.sh <your command>
    """
    print help % dict(project=project, venv=venv, root=root)


def main(argv):
    root = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))

    if os.environ.get('tools_path'):
        root = os.environ['tools_path']
    venv = os.path.join(root, '.venv')
    if os.environ.get('venv'):
        venv = os.environ['venv']

    pip_requires = os.path.join(root, 'requirements.txt')
    test_requires = os.path.join(root, 'test-requirements.txt')
    py_version = "python%s.%s" % (sys.version_info[0], sys.version_info[1])
    setup_cfg = ConfigParser.ConfigParser()
    setup_cfg.read('setup.cfg')
    project = setup_cfg.get('metadata', 'name')

    install = install_venv.InstallVenv(
        root, venv, pip_requires, test_requires, py_version, project)
    options = install.parse_args(argv)
    install.check_python_version()
    install.check_dependencies()
    install.create_virtualenv(no_site_packages=options.no_site_packages)
    install.install_dependencies()
    install.post_process()
    print_help(project, venv, root)

if __name__ == '__main__':
    main(sys.argv)
